------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 1998 Christoph Karl Walter Grein
------------------------------------------------------------------------

with Lexer.Read_File;  pragma Elaborate_All (Lexer);
use  Lexer;

function Count_Ada_Statements (File_Name: in String) return Natural is

  --====================================================================
  -- Authors   William A. Whitaker WIS JPMO (Ada Software Repository),
  --           Christoph Grein, Kai Grambow
  -- Version   2.0
  -- Date      25 October 1998
  --====================================================================
  -- Calculate the Lines of Code (LOCs) of a valid Ada fragment.
  --
  -- A LOC is defined by a semicolon terminator outside of comments,
  -- parentheses, of string or character literals. This definition is
  -- insensitive to formatting or layout of the source.
  --
  -- This source code of function Count_Ada_Statements has by definition
  -- 18 lines.
  -- It has 75 physical lines in its formatted form including 28 lines
  -- containing only comment and 19 blank lines.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  W.W.                    Original
  --  C.G.   1.0  18.05.1990  adapted to our use
  --  K.G.   1.1  14.11.1990  Close file
  --  C.G.   2.0  25.10.1998  Redsign with Lexer
  --====================================================================

  Lines: Natural := 0;
  Level: Natural;

begin

  Initialize (Lexer.Read_File (File_Name), Size_of_Tab => 2);
  -- This is a storage leak, since the file pointer is not deallocated.
  -- The program is however considered to be used stand-alone.

  loop

    case Name (Get_Token) is

       when Left_Parenthesis_AJ =>  -- Every ';' within is in a formal parameter list.

        Level := 1;  -- Count the number of levels of parentheses.

        while Level > 0 loop  -- Read ahead until the whole construct
                              -- is closed, Level = 0.
          case Name (Get_Token) is
            when Left_Parenthesis_AJ  => Level := Level + 1;
            when Right_Parenthesis_AJ => Level := Level - 1;
            when others               => null;
          end case;

        end loop;

      when Semicolon_AJ =>  -- Any ';' that can be found at this point must
                            -- be a valid line of code terminator.
        Lines := Lines + 1;

      when others =>
        null;

    end case;

  end loop;

exception

  when End_Error =>  -- End of file reached.

    return Lines;

end Count_Ada_Statements;
