------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 1998 Christoph Karl Walter Grein
------------------------------------------------------------------------

with Ada.Strings.Fixed;

package HTML is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.2
  -- Date      27 October 1998
  --====================================================================
  -- Constants describing the HyperText Markup Language format (HTML).
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  04.07.1998
  --  C.G.    1.1  21.07.1998 Set background color; add more colors
  --  C.G.    1.2  27.10.1998 New colour names
  --====================================================================

  Header: constant String :=
    "<html><head>"     &
    "<title>"          &  -- title string here
    "</title></head>"  &
    "<body Text=""#000000"" BgColor=""#FFFFFF"">" &
    "<PRE>";

  Footer: constant String := "</PRE></body></html>";

  Title_Index: constant Positive :=
    Ada.Strings.Fixed.Index (Source  => Header,
                             Pattern => "</title>") - 1;

  -- Hexadecimal RGB color names and numbers according to:
  --   HTML 4.0 Reference
  --   Liam Quinn <liam@htmlhelp.com>
  --   http://www.htmlhelp.com

  Black         : constant String := "<FONT COLOR=""#000000"">";  -- HTML 4.0
  Dark_Gray     : constant String := "<FONT COLOR=""#404040"">";
  Gray          : constant String := "<FONT COLOR=""#808080"">";  -- HTML 4.0
  Silver        : constant String := "<FONT COLOR=""#C0C0C0"">";  -- HTML 4.0
  Light_Gray    :          String renames Silver;
  Blue          : constant String := "<FONT COLOR=""#0000FF"">";  -- HTML 4.0
  Aqua          : constant String := "<FONT COLOR=""#00FFFF"">";  -- HTML 4.0
  Cyan          :          String renames Aqua;
  Navy          : constant String := "<FONT COLOR=""#000080"">";  -- HTML 4.0
  Green         : constant String := "<FONT COLOR=""#008000"">";  -- HTML 4.0
  Lime          : constant String := "<FONT COLOR=""#00FF00"">";  -- HTML 4.0
  Teal          : constant String := "<FONT COLOR=""#008080"">";  -- HTML 4.0 (*)
  Maroon        : constant String := "<FONT COLOR=""#800000"">";  -- HTML 4.0
  Purple        : constant String := "<FONT COLOR=""#800080"">";  -- HTML 4.0
  Olive         : constant String := "<FONT COLOR=""#808000"">";  -- HTML 4.0
  Fuchsia       : constant String := "<FONT COLOR=""#FF00FF"">";  -- HTML 4.0
  Magenta       :          String renames Fuchsia;
  Orange        : constant String := "<FONT COLOR=""#FFC800"">";
  Pink          : constant String := "<FONT COLOR=""#FFAFAF"">";
  Red           : constant String := "<FONT COLOR=""#FF0000"">";  -- HTML 4.0
  Yellow        : constant String := "<FONT COLOR=""#FFFF00"">";  -- HTML 4.0
  White         : constant String := "<FONT COLOR=""#FFFFFF"">";  -- HTML 4.0
  No_Color      : constant String := "</FONT>";
  Bold          : constant String := "<B>";
  No_Bold       : constant String := "</B>";
  Italic        : constant String := "<I>";
  No_Italic     : constant String := "</I>";
  Bold_Italic   : constant String := "<B><I>";
  No_Bold_Italic: constant String := "</B></I>";

  -- (*) According to Webster's, in addition to the duck, teal is
  --     "a dark grayish or greenish blue".

  -- These characters have to be replaced by special strings:

  Ampersand   : aliased constant String := "&amp;";
  Less_Than   : aliased constant String := "&lt;";
  Greater_Than: aliased constant String := "&gt;";

  type Replacement_Characters is access constant String;

  type Replacement is record
    Source: Character;
    Target: Replacement_Characters;
  end record;

  Replace: constant array (1 .. 3) of Replacement :=
    (1 => (Source => '&',
           Target => Ampersand'Access),
     2 => (Source => '<',
           Target => Less_Than'Access),
     3 => (Source => '>',
           Target => Greater_Than'Access));

end HTML;
