------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 1998 Christoph Karl Walter Grein
------------------------------------------------------------------------

with Count_Ada_Statements;

with Ada.Command_Line;
with Ada.Text_IO, Ada.IO_Exceptions;

procedure LOCs is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   2.0
  -- Date      17 December 1996
  --====================================================================
  -- Calculate the Lines of Code (LOCs) of valid Ada fragments in a
  -- given set of files.
  -- The files are given as command line arguments.
  -- Syntax: LOCs {list_of_file_names}
  --
  -- Ideas for future improvements:
  -- An option -n could be defined producing output in form of a table
  -- where n is the number of colums for the numeric value.
  -- Example output table for the following command:
  -- LOCs -3 Count_Ada_Statements.adb LOCs.adb LOCs.ads
  --   9 LOCs.adb
  --   - LOCs.ads (Error: does not exist)
  --  32 Count_Ada_Statements.adb
  -- Syntax: LOCs [-tn] {list_of_file_names}
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.   1.0  18.05.1990
  --  C.G.   2.0  17.12.1996  Ada 95; list of file names allowed
  --====================================================================

begin

  for File_Number in 1 .. Ada.Command_Line.Argument_Count loop

    declare
      File_Name: constant String := Ada.Command_Line.Argument (File_Number);
    begin
      Ada.Text_IO.Put_Line
        ("File " & File_Name & " has" &
         Integer'Image (Count_Ada_Statements (File_Name)) &
         " lines of code");
    exception
      when Ada.IO_Exceptions.Name_Error =>
        Ada.Text_IO.Put_Line
          ("Error: File " & File_Name & " does not exist.");
    end;

  end loop;

end LOCs;
