------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 1998 Christoph Karl Walter Grein
------------------------------------------------------------------------

with Ada.Characters.Latin_1;

package RTF is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.3
  -- Date      28 October 1998
  --====================================================================
  -- Constants describing the Rich Text Format (RTF).
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  28.06.1998 Created following a file by Dr. Martin C.
  --                          Carlisle provided with Ada Gnat Integrated
  --                          Development Environment (AdaGIDE)
  --  C.G.    1.1  07.07.1998 Added special characters
  --  C.G.    1.2  23.07.1998 Add more colors
  --  C.G.    1.3  28.10.1998 New colour names
  --====================================================================

  Header: constant String :=
    "{\rtf1\ansi\deff0\deftab720"          &
    "{\fonttbl{\f0\fnil Courier New;}}"    &
    "{\colortbl\red000\green000\blue000;"  &  -- black (no color)
              "\red064\green064\blue064;"  &  -- dark grey
              "\red128\green128\blue128;"  &  -- gray
              "\red192\green192\blue192;"  &  -- silver (light grey)
              "\red000\green000\blue255;"  &  -- blue
              "\red000\green255\blue255;"  &  -- aqua (cyan)
              "\red000\green000\blue128;"  &  -- navy
              "\red000\green128\blue000;"  &  -- green
              "\red000\green255\blue000;"  &  -- lime
              "\red000\green128\blue128;"  &  -- teal
              "\red128\green000\blue000;"  &  -- maroon
              "\red128\green000\blue128;"  &  -- purple
              "\red128\green128\blue000;"  &  -- olive
              "\red255\green000\blue255;"  &  -- fuchsia
              "\red255\green200\blue000;"  &  -- orange
              "\red255\green175\blue175;"  &  -- pink
              "\red255\green000\blue000;"  &  -- red
              "\red255\green255\blue000;"  &  -- yellow
              "\red255\green255\blue255;}" &  -- white
    "\deflang1033\pard\f0\fs20";

  -- The colour names are the same as in HTML.
  
  Footer: constant String := "\par }";

  Black      : constant String := "\cf0 ";
  Dark_Grey  : constant String := "\cf1 ";
  Gray       : constant String := "\cf2 ";
  Silver     : constant String := "\cf3 ";
  Light_Gray :          String renames Silver;
  Blue       : constant String := "\cf4 ";
  Aqua       : constant String := "\cf5 ";
  Cyan       :          String renames Aqua;
  Navy       : constant String := "\cf6 ";
  Green      : constant String := "\cf7 ";
  Lime       : constant String := "\cf8 ";
  Teal       : constant String := "\cf9 ";
  Maroon     : constant String := "\cf10 ";
  Purple     : constant String := "\cf11 ";
  Olive      : constant String := "\cf12 ";
  Fuchsia    : constant String := "\cf13 ";
  Magenta    :          String renames Fuchsia;
  Orange     : constant String := "\cf14 ";
  Pink       : constant String := "\cf15 ";
  Red        : constant String := "\cf16 ";
  Yellow     : constant String := "\cf17 ";
  White      : constant String := "\cf18 ";
  No_Color   :          String renames Black;
  Bold       : constant String := "\b ";
  Bold_Italic: constant String := "\b\i ";
  Italic     : constant String := "\i ";
  Plain      : constant String := "\plain\fs20 ";

  BOL: constant String := "\par ";  -- Begin Of Line (to be put after Line Feed)

  -- These characters have to be replaced by special strings:

  Tab        : aliased constant String := "\tab ";  -- tabulation
  Back_Slash : aliased constant String := "\\";
  Left_Brace : aliased constant String := "\{";
  Right_Brace: aliased constant String := "\}";

  type Replacement_Characters is access constant String;

  type Replacement is record
    Source: Character;
    Target: Replacement_Characters;
  end record;

  Replace: constant array (1 .. 4) of Replacement :=
    (1 => (Source => Ada.Characters.Latin_1.HT,
           Target => Tab'Access),
     2 => (Source => '\',
           Target => Back_Slash'Access),
     3 => (Source => '{',
           Target => Left_Brace'Access),
     4 => (Source => '}',
           Target => Right_Brace'Access));

end RTF;
