------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 1998 Christoph Karl Walter Grein
------------------------------------------------------------------------

separate (Test_Lexer)

procedure Test_Ada is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.1
  -- Date      22 September 1998
  --====================================================================
  -- Test program for Ada Lexer.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  26.07.1998 Made separate; Java
  --  C.G.    1.1  22.09.1998 Replace Ampersand_AJ and Vertical_Bar_AJ
  --====================================================================

  procedure Test (Testee: in String; Expected: in Expectation) is
  begin
    Test (Testee, Ada_Language, Expected);
  end Test;

begin

  Ada.Text_IO.Put_Line ("Ada test");
  Ada.Text_IO.Put_Line ("========");
  Ada.Text_IO.New_Line;

  Test (Testee   => "{[$%^~?!_]}",  -- Java characters
        Expected => (1 => (Lexer.Bad_Token_AJ, + "{", Lexer.Non_Language_Character),
                     2 => (Lexer.Bad_Token_AJ, + "[", Lexer.Non_Language_Character),
                     3 => (Lexer.Bad_Token_AJ, + "$", Lexer.Non_Language_Character),
                     4 => (Lexer.Bad_Token_AJ, + "%", Lexer.Non_Language_Character),
                     5 => (Lexer.Bad_Token_AJ, + "^", Lexer.Non_Language_Character),
                     6 => (Lexer.Bad_Token_AJ, + "~", Lexer.Non_Language_Character),
                     7 => (Lexer.Bad_Token_AJ, + "?", Lexer.Non_Language_Character),
                     8 => (Lexer.Bad_Token_AJ, + "!", Lexer.Non_Language_Character),
                     9 => (Lexer.Bad_Token_AJ, + "_", Lexer.Non_Language_Character),
                    10 => (Lexer.Bad_Token_AJ, + "]", Lexer.Non_Language_Character),
                    11 => (Lexer.Bad_Token_AJ, + "}", Lexer.Non_Language_Character)));

  Test (Testee   => "     -- an Ada comment, not a Java decrement",
        Expected => (1 => (Lexer.Comment_A, + "-- an Ada comment, not a Java decrement")));

  Test (Testee   => "=> .. ** := /= >= <= << >> <> ++ == != >>> || && " &  -- Ada and Java
                    "+= -= *= &= |= ^= %= <<= >>= >>>=",                   -- delimiters and operators
        Expected => (1 => (Lexer.Arrow_A              , + "=>"),
                     2 => (Lexer.Double_Dot_A         , + ".."),
                     3 => (Lexer.Exponentiate_A       , + "**"),
                     4 => (Lexer.Assignment_A         , + ":="),
                     5 => (Lexer.Not_Equal_A          , + "/="),
                     6 => (Lexer.Greater_Equal_AJ     , + ">="),
                     7 => (Lexer.Less_Equal_AJ        , + "<="),
                     8 => (Lexer.Left_Label_Bracket_A , + "<<"),
                     9 => (Lexer.Right_Label_Bracket_A, + ">>"),
                    10 => (Lexer.Box_A                , + "<>"),
                    11 => (Lexer.Plus_AJ              , + "+"),
                    12 => (Lexer.Plus_AJ              , + "+"),
                    13 => (Lexer.Equal_A              , + "="),
                    14 => (Lexer.Equal_A              , + "="),
                    15 => (Lexer.Bad_Token_AJ         , + "!", Lexer.Non_Language_Character),
                    16 => (Lexer.Equal_A              , + "="),
                    17 => (Lexer.Right_Label_Bracket_A, + ">>"),
                    18 => (Lexer.Greater_Than_AJ      , + ">"),
                    19 => (Lexer.Alternative_A        , + "|"),
                    20 => (Lexer.Alternative_A        , + "|"),
                    21 => (Lexer.Concatenate_A        , + "&"),
                    22 => (Lexer.Concatenate_A        , + "&"),
                    23 => (Lexer.Plus_AJ              , + "+"),
                    24 => (Lexer.Equal_A              , + "="),
                    25 => (Lexer.Minus_AJ             , + "-"),
                    26 => (Lexer.Equal_A              , + "="),
                    27 => (Lexer.Times_AJ             , + "*"),
                    28 => (Lexer.Equal_A              , + "="),
                    29 => (Lexer.Concatenate_A        , + "&"),
                    30 => (Lexer.Equal_A              , + "="),
                    31 => (Lexer.Alternative_A        , + "|"),
                    32 => (Lexer.Equal_A              , + "="),
                    33 => (Lexer.Bad_Token_AJ         , + "^", Lexer.Non_Language_Character),
                    34 => (Lexer.Equal_A              , + "="),
                    35 => (Lexer.Bad_Token_AJ         , + "%", Lexer.Non_Language_Character),
                    36 => (Lexer.Equal_A              , + "="),
                    37 => (Lexer.Left_Label_Bracket_A , + "<<"),
                    38 => (Lexer.Equal_A              , + "="),
                    39 => (Lexer.Right_Label_Bracket_A, + ">>"),
                    40 => (Lexer.Equal_A              , + "="),
                    41 => (Lexer.Right_Label_Bracket_A, + ">>"),
                    42 => (Lexer.Greater_Equal_AJ     , + ">=")));

  Test (Testee   => "Identifier",
        Expected => (1 => (Lexer.Identifier_AJ, + "Identifier")));
  Test (Testee   => "Ident_ifier",
        Expected => (1 => (Lexer.Identifier_AJ, + "Ident_ifier")));
  Test (Testee   => "Ident__ifier",
        Expected => (1 => (Lexer.Bad_Token_AJ , + "Ident__ifier", Lexer.Illegal_Underline)));
  Test (Testee   => "Identifier_",
        Expected => (1 => (Lexer.Bad_Token_AJ , + "Identifier_" , Lexer.Illegal_Underline)));
  Test (Testee   => "_Identifier",
        Expected => (1 => (Lexer.Bad_Token_AJ , + "_Identifier" , Lexer.Illegal_Underline)));

  Test (Testee   => "A.a",
        Expected => (1 => (Lexer.Identifier_AJ, + "A"),
                     2 => (Lexer.Dot_AJ       , + "."),
                     3 => (Lexer.Identifier_AJ, + "a")));
  Test (Testee   => "A . a",
        Expected => (1 => (Lexer.Identifier_AJ, + "A"),
                     2 => (Lexer.Dot_AJ       , + "."),
                     3 => (Lexer.Identifier_AJ, + "a")));
  Test (Testee   => "A..a",
        Expected => (1 => (Lexer.Identifier_AJ, + "A"),
                     2 => (Lexer.Double_Dot_A , + ".."),
                     3 => (Lexer.Identifier_AJ, + "a")));

  Test (Testee   => "'''",
        Expected => (1 => (Lexer.Character_AJ, + "'''")));
  Test (Testee   => "''",
        Expected => (1 => (Lexer.Tick_A      , + "'"),
                     2 => (Lexer.Tick_A      , + "'")));

  Test (Testee   => """""",
        Expected => (1 => (Lexer.String_AJ   , + """""")));
  Test (Testee   => """xx""",
        Expected => (1 => (Lexer.String_AJ   , + """xx""")));
  Test (Testee   => """xx",
        Expected => (1 => (Lexer.Bad_Token_AJ, + """xx"               , Lexer.Missing_String_Quote)));
  Test (Testee   => """xx" & HT & "xx""",
        Expected => (1 => (Lexer.Bad_Token_AJ, +("""xx" & HT & "xx"""), Lexer.Illegal_Literal)));

  -- Integers without exponents

  Test (Testee   => "123",
        Expected => (1 => (Lexer.Integer_AJ  , + "123")));
  Test (Testee   => "123_45",
        Expected => (1 => (Lexer.Integer_AJ  , + "123_45")));
  Test (Testee   => "12__3",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "12__3", Lexer.Illegal_Underline)));
  Test (Testee   => "_123",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "_123" , Lexer.Illegal_Underline)));
  Test (Testee   => "123_",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123_" , Lexer.Illegal_Underline)));

  Test (Testee   => "123a",
        Expected => (1 => (Lexer.Integer_AJ   , + "123"),
                     2 => (Lexer.Identifier_AJ, + "a")));
  Test (Testee   => "123L",
        Expected => (1 => (Lexer.Integer_AJ   , + "123"),  -- Java long integer
                     2 => (Lexer.Identifier_AJ, + "L")));

  -- Reals without exponents

  Test (Testee   => "123.45",
        Expected => (1 => (Lexer.Real_AJ     , + "123.45")));
  Test (Testee   => "123.4_5",
        Expected => (1 => (Lexer.Real_AJ     , + "123.4_5")));
  Test (Testee   => "123._45",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123._45" , Lexer.Illegal_Underline)));
  Test (Testee   => "123.4__5",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123.4__5", Lexer.Illegal_Underline)));
  Test (Testee   => "123.45_",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123.45_" , Lexer.Illegal_Underline)));
  Test (Testee   => "_123.45",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "_123.45" , Lexer.Illegal_Underline)));
  Test (Testee   => "123.",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123."    , Lexer.Illegal_Literal)));

  Test (Testee   => "123.0a",
        Expected => (1 => (Lexer.Real_AJ      , + "123.0"),
                     2 => (Lexer.Identifier_AJ, + "a")));
  Test (Testee   => "123.0f",
        Expected => (1 => (Lexer.Real_AJ      , + "123.0"),  -- Java float
                     2 => (Lexer.Identifier_AJ, + "f")));
  Test (Testee   => "123.0D",
        Expected => (1 => (Lexer.Real_AJ      , + "123.0"),  -- Java double
                     2 => (Lexer.Identifier_AJ, + "D")));
  Test (Testee   => "123.4.0",
        Expected => (1 => (Lexer.Real_AJ      , + "123.4"),
                     2 => (Lexer.Dot_AJ       , + "."),
                     3 => (Lexer.Integer_AJ   , + "0")));

  -- A range

  Test (Testee   => "123..45",
        Expected => (1 => (Lexer.Integer_AJ  , + "123"),
                     2 => (Lexer.Double_Dot_A, + ".."),
                     3 => (Lexer.Integer_AJ  , + "45")));

  -- Integers with exponents

  Test (Testee   => "123E4",
        Expected => (1 => (Lexer.Integer_AJ  , + "123E4")));
  Test (Testee   => "123e+4",
        Expected => (1 => (Lexer.Integer_AJ  , + "123e+4")));
  Test (Testee   => "123E-4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123E-4", Lexer.Illegal_Literal)));
  Test (Testee   => "123E",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123E"  , Lexer.Illegal_Literal)));
  Test (Testee   => "123E+",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123E+" , Lexer.Illegal_Literal)));

  Test (Testee   => "123E_4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123E_4"  , Lexer.Illegal_Underline)));
  Test (Testee   => "123E+_4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123E+_4" , Lexer.Illegal_Underline)));
  Test (Testee   => "123E-_4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123E-_4" , Lexer.Illegal_Literal)));
  Test (Testee   => "123E4__4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123E4__4", Lexer.Illegal_Underline)));
  Test (Testee   => "123E4_",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123E4_"  , Lexer.Illegal_Underline)));

  Test (Testee   => "123E4a",
        Expected => (1 => (Lexer.Integer_AJ   , + "123E4"),
                     2 => (Lexer.Identifier_AJ, + "a")));
  Test (Testee   => "123Ea",
        Expected => (1 => (Lexer.Bad_Token_AJ , + "123E", Lexer.Illegal_Literal),
                     2 => (Lexer.Identifier_AJ, + "a")));

  -- Reals with exponents

  Test (Testee   => "123.4E5",
        Expected => (1 => (Lexer.Real_AJ     , + "123.4E5")));
  Test (Testee   => "123.4_E5",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123.4_E5", Lexer.Illegal_Underline)));
  Test (Testee   => "123.4E+5",
        Expected => (1 => (Lexer.Real_AJ     , + "123.4E+5")));
  Test (Testee   => "123.E-5",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "123.E-5", Lexer.Illegal_Literal)));

  Test (Testee   => "123.4E5.0",
        Expected => (1 => (Lexer.Real_AJ   , + "123.4E5"),
                     2 => (Lexer.Dot_AJ    , + "."),
                     3 => (Lexer.Integer_AJ, + "0")));

  -- A range

  Test (Testee   => "123e0..45.6e-7",
        Expected => (1 => (Lexer.Integer_AJ  , + "123e0"),
                     2 => (Lexer.Double_Dot_A, + ".."),
                     3 => (Lexer.Real_AJ     , + "45.6e-7")));

  -- Based numbers

  Test (Testee   => " 3#+",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "3#", Lexer.Missing_Base_Quote),
                     2 => (Lexer.Plus_AJ     , + "+")));

  -- Based integers without exponents

  Test (Testee   => " 3#12#",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "3#12#"   , Base => 3)));
  Test (Testee   => " 3#13#",
        Expected => (1 => (Lexer.Bad_Token_AJ    , + "3#13#"   , Lexer.Illegal_Extended_Digit)));
  Test (Testee   => "16#ef#",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "16#ef#"  , Base => 16)));
  Test (Testee   => "15#ef#",
        Expected => (1 => (Lexer.Bad_Token_AJ    , + "15#ef#"  , Lexer.Illegal_Extended_Digit)));
  Test (Testee   => "16#e_f#",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "16#e_f#" , Base => 16)));
  Test (Testee   => "16#_EF#",
        Expected => (1 => (Lexer.Bad_Token_AJ    , + "16#_EF#" , Lexer.Illegal_Underline)));
  Test (Testee   => "16#E__F#",
        Expected => (1 => (Lexer.Bad_Token_AJ    , + "16#E__F#", Lexer.Illegal_Underline)));
  Test (Testee   => "16#EF_#",
        Expected => (1 => (Lexer.Bad_Token_AJ    , + "16#EF_#" , Lexer.Illegal_Underline)));

  Test (Testee   => "10#123+",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "10#123", Lexer.Missing_Base_Quote),
                     2 => (Lexer.Plus_AJ     , + "+")));
  Test (Testee   => "10#123..",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "10#123", Lexer.Missing_Base_Quote),
                     2 => (Lexer.Double_Dot_A, + "..")));

  Test (Testee   => "0123L",
        Expected => (1 => (Lexer.Integer_AJ   , + "0123"),  -- Java octal long integer
                     2 => (Lexer.Identifier_AJ, + "L")));
  Test (Testee   => "0x123L",
        Expected => (1 => (Lexer.Integer_AJ   , + "0"),  -- Java hexadecimal long integer
                     2 => (Lexer.Identifier_AJ, + "x123L")));

  -- Based integers with exponents

  Test (Testee   => "10#123#E4",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "10#123#E4" , Base => 10)));
  Test (Testee   => "10#123#E+4",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "10#123#E+4", Base => 10)));
  Test (Testee   => "10#123#E-4",
        Expected => (1 => (Lexer.Bad_Token_AJ    , + "10#123#E-4", Lexer.Illegal_Literal)));
  Test (Testee   => "10#123#e4A",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "10#123#e4" , Base => 10),
                     2 => (Lexer.Identifier_AJ   , + "A")));
  Test (Testee   => "10#123#E+4A",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "10#123#E+4", Base => 10),
                     2 => (Lexer.Identifier_AJ   , + "A")));
  Test (Testee   => "10#123#E-a",
        Expected => (1 => (Lexer.Bad_Token_AJ    , + "10#123#E-", Lexer.Illegal_Literal),
                     2 => (Lexer.Identifier_AJ   , + "a")));

  Test (Testee   => "10#123#E4-a",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "10#123#E4", Base => 10),
                     2 => (Lexer.Minus_AJ        , + "-"),
                     3 => (Lexer.Identifier_AJ   , + "a")));

  Test (Testee   => "10#123#E4#",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "10#123#E4", Base => 10),
                     2 => (Lexer.Bad_Token_AJ    , + "#"        , Lexer.Non_Language_Character)));
  Test (Testee   => "10#12#3#E4",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "10#12#"   , Base => 10),
                     2 => (Lexer.Bad_Token_AJ    , + "3#E4"     , Lexer.Missing_Base_Quote)));

  Test (Testee   => "20#123#E4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "20#123#E4", Lexer.Illegal_Base)));
  Test (Testee   => "1#123#E4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "1#123#E4" , Lexer.Illegal_Base)));

  -- Based reals without exponents

  Test (Testee   => " 3#12.0#",
        Expected => (1 => (Lexer.Based_Real_A, +  "3#12.0#"  , Base => 3)));
  Test (Testee   => " 3#45.0#",
        Expected => (1 => (Lexer.Bad_Token_AJ, +  "3#45.0#"  , Lexer.Illegal_Extended_Digit)));
  Test (Testee   => "16#ab.ef#",
        Expected => (1 => (Lexer.Based_Real_A, + "16#ab.ef#" , Base => 16)));
  Test (Testee   => "15#ab.ef#",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "15#ab.ef#" , Lexer.Illegal_Extended_Digit)));
  Test (Testee   => "16#ef_.ab#",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "16#ef_.ab#", Lexer.Illegal_Underline)));
  Test (Testee   => "16#ef.#",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "16#ef.#"   , Lexer.Illegal_Literal)));

  Test (Testee   => "10#123.+",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "10#123." , Lexer.Missing_Base_Quote),
                     2 => (Lexer.Plus_AJ     , + "+")));
  Test (Testee   => "10#123.4.",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "10#123.4", Lexer.Missing_Base_Quote),
                     2 => (Lexer.Dot_AJ      , + ".")));

  Test (Testee   => "10#.4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "10#.4" , Lexer.Missing_Base_Quote)));
  Test (Testee   => "10#.4#",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "10#.4#", Lexer.Illegal_Literal)));

  -- Based reals with exponents

  Test (Testee   => " 3#12.0#e5",
        Expected => (1 => (Lexer.Based_Real_A, +  "3#12.0#e5"   , Base => 3)));
  Test (Testee   => " 3#12.0#e+5",
        Expected => (1 => (Lexer.Based_Real_A, +  "3#12.0#e+5"  , Base => 3)));
  Test (Testee   => " 3#12.0#E-5",
        Expected => (1 => (Lexer.Based_Real_A, +  "3#12.0#E-5"  , Base => 3)));
  Test (Testee   => " 3#45.0#e+1",
        Expected => (1 => (Lexer.Bad_Token_AJ, +  "3#45.0#e+1"  , Lexer.Illegal_Extended_Digit)));
  Test (Testee   => "16#ab.ef#e-1",
        Expected => (1 => (Lexer.Based_Real_A, + "16#ab.ef#e-1" , Base => 16)));
  Test (Testee   => "15#ab.ef#e1",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "15#ab.ef#e1"  , Lexer.Illegal_Extended_Digit)));
  Test (Testee   => "16#ef_.ab#E-4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "16#ef_.ab#E-4", Lexer.Illegal_Underline)));

  Test (Testee   => "10#123.#e-7+",
        Expected => (1 => (Lexer.Bad_Token_AJ , + "10#123.#e-7", Lexer.Illegal_Literal),
                     2 => (Lexer.Plus_AJ      , + "+")));
  Test (Testee   => "10#123.4#e.",
        Expected => (1 => (Lexer.Bad_Token_AJ , + "10#123.4#e" , Lexer.Illegal_Literal),
                     2 => (Lexer.Dot_AJ       , + ".")));
  Test (Testee   => "10#123.0#E-4-a",
        Expected => (1 => (Lexer.Based_Real_A , + "10#123.0#E-4", Base => 10),
                     2 => (Lexer.Minus_AJ     , + "-"),
                     3 => (Lexer.Identifier_AJ, + "a")));

  Test (Testee   => "16##",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "16##"    , Lexer.Illegal_Literal)));
  Test (Testee   => "16##e-4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "16##e-4" , Lexer.Illegal_Literal)));

  Test (Testee   => "10#.4#e3",
        Expected => (1 => (Lexer.Bad_Token_AJ , + "10#.4#e3", Lexer.Illegal_Literal)));
  Test (Testee   => "10#.#e-3",
        Expected => (1 => (Lexer.Bad_Token_AJ , + "10#.#e-3", Lexer.Illegal_Literal)));
  Test (Testee   => "10#..#e-3",
        Expected => (1 => (Lexer.Bad_Token_AJ , + "10#"     , Lexer.Missing_Base_Quote),
                     2 => (Lexer.Double_Dot_A , + ".."),
                     3 => (Lexer.Bad_Token_AJ , + "#"       , Lexer.Non_Language_Character),
                     4 => (Lexer.Identifier_AJ, + "e"),
                     5 => (Lexer.Minus_AJ     , + "-"),
                     6 => (Lexer.Integer_AJ   , + "3")));

  -- Test with end of line (Windows CR, LF; Unix LF)

  Test (Testee   => ''' & CR & LF & ''',
        Expected => (1 => (Lexer.Tick_A, + "'"),
                     2 => (Lexer.Tick_A, + "'")));
  Test (Testee   => ''' & LF & ''',
        Expected => (1 => (Lexer.Tick_A, + "'"),
                     2 => (Lexer.Tick_A, + "'")));

  Test (Testee   => """a string""" & CR & LF & '+',
        Expected => (1 => (Lexer.String_AJ, + """a string"""),
                     2 => (Lexer.Plus_AJ  , + "+")));
  Test (Testee   => """a string""" & LF & '+',
        Expected => (1 => (Lexer.String_AJ, + """a string"""),
                     2 => (Lexer.Plus_AJ  , + "+")));

  Test (Testee   => """a string with missing end quote" & CR & LF & '+',
        Expected => (1 => (Lexer.Bad_Token_AJ, + """a string with missing end quote", Lexer.Missing_String_Quote),
                     2 => (Lexer.Plus_AJ     , + "+")));
  Test (Testee   => """a string with missing end quote" & LF & '+',
        Expected => (1 => (Lexer.Bad_Token_AJ, + """a string with missing end quote", Lexer.Missing_String_Quote),
                     2 => (Lexer.Plus_AJ     , + "+")));

  Test (Testee   => "-- a comment" & CR & LF & '+',
        Expected => (1 => (Lexer.Comment_A   , + "-- a comment"),
                     2 => (Lexer.Plus_AJ     , + "+")));
  Test (Testee   => "-- a comment" & LF & '+',
        Expected => (1 => (Lexer.Comment_A   , + "-- a comment"),
                     2 => (Lexer.Plus_AJ     , + "+")));

  Test (Testee   => "First_Identifier" & CR & LF & "Second_Identifier",
        Expected => (1 => (Lexer.Identifier_AJ, + "First_Identifier"),
                     2 => (Lexer.Identifier_AJ, + "Second_Identifier")));
  Test (Testee   => "First_Identifier" &  LF & "Second_Identifier",
        Expected => (1 => (Lexer.Identifier_AJ, + "First_Identifier"),
                     2 => (Lexer.Identifier_AJ, + "Second_Identifier")));

  Test (Testee => "12#1b#e4" & CR & LF & "13#bC.a#e-4",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "12#1b#e4"   , Base => 12),
                     2 => (Lexer.Based_Real_A    , + "13#bC.a#e-4", Base => 13)));
  Test (Testee => "12#1b#e4" & LF & "13#bC.a#e-4",
        Expected => (1 => (Lexer.Based_Integer_AJ, + "12#1b#e4"   , Base => 12),
                     2 => (Lexer.Based_Real_A    , + "13#bC.a#e-4", Base => 13)));

  Test (Testee => "12e" & CR & LF & "13.0e-4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "12e"    , Lexer.Illegal_Literal),
                     2 => (Lexer.Real_AJ     , + "13.0e-4")));
  Test (Testee => "12e" & LF & "13.0e-4",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "12e"    , Lexer.Illegal_Literal),
                     2 => (Lexer.Real_AJ     , + "13.0e-4")));

end Test_Ada;
