------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 1998 Christoph Karl Walter Grein
------------------------------------------------------------------------

separate (Test_Lexer)

procedure Test_Java is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.3
  -- Date      23 September 1998
  --====================================================================
  -- Test program for Java Lexer. Need only test Java specific tokens.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  30.07.1998 No test for special Java characters in
  --                          string literals, no test for lazy reals
  --  C.G.    1.1  05.08.1998 Added Documentation_Tag and character
  --                          literals with simple escape sequences
  --  C.G.    1.2  17.09.1998 Added octal escape characters,
  --                          based integers and lazy reals
  --  C.G.    1.3  23.09.1998 Replace Ampersand_AJ and Vertical_Bar_AJ
  --====================================================================

  procedure Test (Testee: in String; Expected: in Expectation) is
  begin
    Test (Testee, Java_Language, Expected);
  end Test;

begin

  Ada.Text_IO.Put_Line ("Java test");
  Ada.Text_IO.Put_Line ("=========");
  Ada.Text_IO.New_Line;

  Test (Testee   => "{[%^~?!&|]}",  -- Java characters
        Expected => (1 => (Lexer.LeftBrace_J   , + "{"),
                     2 => (Lexer.LeftBracket_J , + "["),
                     3 => (Lexer.Remainder_J   , + "%"),
                     4 => (Lexer.Xor_J         , + "^"),
                     5 => (Lexer.Complement_J  , + "~"),
                     6 => (Lexer.Conditional_J , + "?"),
                     7 => (Lexer.Not_J         , + "!"),
                     8 => (Lexer.And_J         , + "&"),
                     9 => (Lexer.Or_J          , + "|"),
                    10 => (Lexer.RightBracket_J, + "]"),
                    11 => (Lexer.RightBrace_J  , + "}")));

  Test (Testee   => "=> .. ** := /= >= <= << >> <> ++ -- == != >>> || && " &  -- Ada and Java
                    "+= -= *= /= &= |= ^= %= <<= >>= >>>=",                   -- delimiters and operators
        Expected => (1 => (Lexer.Assignment_J              , + "="),
                     2 => (Lexer.Greater_Than_AJ           , + ">"),
                     3 => (Lexer.Dot_AJ                    , + "."),
                     4 => (Lexer.Dot_AJ                    , + "."),
                     5 => (Lexer.Times_AJ                  , + "*"),
                     6 => (Lexer.Times_AJ                  , + "*"),
                     7 => (Lexer.Colon_AJ                  , + ":"),
                     8 => (Lexer.Assignment_J              , + "="),
                     9 => (Lexer.DivideAssign_J            , + "/="),
                    10 => (Lexer.Greater_Equal_AJ          , + ">="),
                    11 => (Lexer.Less_Equal_AJ             , + "<="),
                    12 => (Lexer.LeftShift_J               , + "<<"),
                    13 => (Lexer.RightShift_J              , + ">>"),
                    14 => (Lexer.Less_Than_AJ              , + "<"),
                    15 => (Lexer.Greater_Than_AJ           , + ">"),
                    16 => (Lexer.Increment_J               , + "++"),
                    17 => (Lexer.Decrement_J               , + "--"),
                    18 => (Lexer.Equal_J                   , + "=="),
                    19 => (Lexer.NotEqual_J                , + "!="),
                    20 => (Lexer.UnsignedRightShift_J      , + ">>>"),
                    21 => (Lexer.ShortCutOr_J              , + "||"),
                    22 => (Lexer.ShortCutAnd_J             , + "&&"),
                    23 => (Lexer.PlusAssign_J              , + "+="),
                    24 => (Lexer.MinusAssign_J             , + "-="),
                    25 => (Lexer.TimesAssign_J             , + "*="),
                    26 => (Lexer.DivideAssign_J            , + "/="),
                    27 => (Lexer.AndAssign_J               , + "&="),
                    28 => (Lexer.OrAssign_J                , + "|="),
                    29 => (Lexer.XorAssign_J               , + "^="),
                    30 => (Lexer.RemainderAssign_J         , + "%="),
                    31 => (Lexer.LeftShiftAssign_J         , + "<<="),
                    32 => (Lexer.RightShiftAssign_J        , + ">>="),
                    33 => (Lexer.UnsignedRightShiftAssign_J, + ">>>=")));

  -- Silly (but legal) identifiers

  Test (Testee   => "_ __ $ $$ $_ _$",
        Expected => (1 => (Lexer.Identifier_AJ, + "_"),
                     2 => (Lexer.Identifier_AJ, + "__" ),
                     3 => (Lexer.Identifier_AJ, + "$"),
                     4 => (Lexer.Identifier_AJ, + "$$"),
                     5 => (Lexer.Identifier_AJ, + "$_"),
                     6 => (Lexer.Identifier_AJ, + "_$")));
  Test (Testee   => "$identifier$ _identifier_",
        Expected => (1 => (Lexer.Identifier_AJ, + "$identifier$"),
                     2 => (Lexer.Identifier_AJ, + "_identifier_")));

  -- Empty comments

  Test (Testee   => "     //",
        Expected => (1 => (Lexer.EndOfLineComment_J  , + "//")));
  Test (Testee   => "     /**/",
        Expected => (1 => (Lexer.TraditionalComment_J, + "/**/")));
  Test (Testee   => "     /*" & LF & "    " & LF & LF & "*/",
        Expected => (1 => (Lexer.CommentHead_J       , + "/*"),
                     2 => (Lexer.CommentTail_J       , + "*/")));
  Test (Testee   => "     /***/",
        Expected => (1 => (Lexer.Documentation_J     , + "/***/", Lexer.No_Tag)));
  Test (Testee   => "     /**" & CR & LF & CR & LF & CR & LF & "*/",
        Expected => (1 => (Lexer.DocumentationHead_J , + "/**"  , Lexer.No_Tag),
                     2 => (Lexer.DocumentationTail_J , + "*/"   , Lexer.No_Tag)));

  -- Incomplete comments

  Test (Testee   => "   /*/   ",
        Expected => (1 => (Lexer.CommentHead_J, + "/*/   ")));

  -- Proper comments

  Test (Testee   => "     // a comment line  ",
        Expected => (1 => (Lexer.EndOfLineComment_J  , + "// a comment line  ")));
  Test (Testee   => "     /* a comment */ identifier",
        Expected => (1 => (Lexer.TraditionalComment_J, + "/* a comment */"),
                     2 => (Lexer.Identifier_AJ       , + "identifier")));
  Test (Testee   => "     /* a multi  " & LF & "   line " & LF & " comment */ identifier",
        Expected => (1 => (Lexer.CommentHead_J       , + "/* a multi  "),
                     2 => (Lexer.CommentBody_J       , + "line "),
                     3 => (Lexer.CommentTail_J       , + "comment */"),
                     4 => (Lexer.Identifier_AJ       , + "identifier")));
  Test (Testee   => "     /** a documentation comment */ identifier",
        Expected => (1 => (Lexer.Documentation_J, + "/** a documentation comment */", Lexer.No_Tag),
                     2 => (Lexer.Identifier_AJ       , + "identifier")));
  Test (Testee   => "     /** a multi  " & CR & LF & "   line " & CR & LF & " @documentation comment */ identifier",
        Expected => (1 => (Lexer.DocumentationHead_J , + "/** a multi  "            , Lexer.No_Tag),
                     2 => (Lexer.DocumentationBody_J , + "line "                    , Lexer.No_Tag),
                     3 => (Lexer.DocumentationTail_J , + "@documentation comment */", Lexer.No_Tag),
                     4 => (Lexer.Identifier_AJ       , + "identifier")));

  Test (Testee   => "     /** @author Me  " & CR & LF & "   @version1 missing blank " & CR & LF & " @see*/ identifier",
        Expected => (1 => (Lexer.DocumentationHead_J , + "/** @author Me  "         , Lexer.Author_Tag),
                     2 => (Lexer.DocumentationBody_J , + "@version1 missing blank " , Lexer.No_Tag),
                     3 => (Lexer.DocumentationTail_J , + "@see*/"                   , Lexer.See_Tag),
                     4 => (Lexer.Identifier_AJ       , + "identifier")));

  Test (Testee   => "/* this comment /* // /** ends here: */",  -- JRM 3.7
        Expected => (1 => (Lexer.TraditionalComment_J, + "/* this comment /* // /** ends here: */")));

  -- Character literals

  Test (Testee   => "'",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "'"   , Lexer.Non_Language_Character)));
  Test (Testee   => "'\'",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "'\'" , Lexer.Non_Language_Character)));
  Test (Testee   => "'''",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "'''" , Lexer.Non_Language_Character)));
  Test (Testee   => "'""'",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "'""'", Lexer.Non_Language_Character)));

  Test (Testee   => "'\b'",  -- back space
        Expected => (1 => (Lexer.Character_AJ, + "'\b'")));
  Test (Testee   => "'\t'",  -- horizontal tabulation
        Expected => (1 => (Lexer.Character_AJ, + "'\t'")));
  Test (Testee   => "'\n'",  -- new line
        Expected => (1 => (Lexer.Character_AJ, + "'\n'")));
  Test (Testee   => "'\f'",  -- form feed
        Expected => (1 => (Lexer.Character_AJ, + "'\f'")));
  Test (Testee   => "'\r'",  -- carriage return
        Expected => (1 => (Lexer.Character_AJ, + "'\r'")));
  Test (Testee   => "'\""'",  -- double quote
        Expected => (1 => (Lexer.Character_AJ, + "'\""'")));
  Test (Testee   => "'\''",  -- single quote
        Expected => (1 => (Lexer.Character_AJ, + "'\''")));
  Test (Testee   => "'\\'",  -- back slash
        Expected => (1 => (Lexer.Character_AJ, + "'\\'")));
  Test (Testee   => "'\g'",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "'\g'", Lexer.Non_Language_Character)));
  Test (Testee   => "'\" & LF & ''',
        Expected => (1 => (Lexer.Bad_Token_AJ, + "'\"  , Lexer.Non_Language_Character),
                     2 => (Lexer.Bad_Token_AJ, + "'"   , Lexer.Non_Language_Character)));

  Test (Testee   => "'\7'",  -- octal bit pattern
        Expected => (1 => (Lexer.Character_AJ, + "'\7'")));
  Test (Testee   => "'\77'",  -- octal bit pattern
        Expected => (1 => (Lexer.Character_AJ, + "'\77'")));
  Test (Testee   => "'\377'",  -- maximal octal bit pattern
        Expected => (1 => (Lexer.Character_AJ, + "'\377'")));

  Test (Testee   => "'\0377'",  -- octal bit pattern too long
        Expected => (1 => (Lexer.Bad_Token_AJ , + "'\037"  , Lexer.Illegal_Literal),
                     2 => (Lexer.Integer_AJ   , + "7"),
                     3 => (Lexer.Bad_Token_AJ , + "'"      , Lexer.Non_Language_Character)));
  Test (Testee   => "'\77 '",  -- no blanks allowed in octal bit pattern
        Expected => (1 => (Lexer.Bad_Token_AJ , + "'\77"   , Lexer.Illegal_Literal),
                     2 => (Lexer.Bad_Token_AJ , + "'"      , Lexer.Non_Language_Character)));
  Test (Testee   => "'\477'",  -- octal bit pattern too big
        Expected => (1 => (Lexer.Bad_Token_AJ , + "'\477'" , Lexer.Illegal_Extended_Digit)));
  Test (Testee   => "'\8'",  -- no octal bit pattern
        Expected => (1 => (Lexer.Bad_Token_AJ , + "'\8'"   , Lexer.Illegal_Extended_Digit)));
  Test (Testee   => "'\78'",  -- no octal bit pattern
        Expected => (1 => (Lexer.Bad_Token_AJ , + "'\78'"  , Lexer.Illegal_Extended_Digit)));
  Test (Testee   => "'\7a7'",  -- no octal bit pattern
        Expected => (1 => (Lexer.Bad_Token_AJ , + "'\7"    , Lexer.Illegal_Literal),
                     2 => (Lexer.Identifier_AJ, + "a7"),
                     3 => (Lexer.Bad_Token_AJ , + "'"      , Lexer.Non_Language_Character)));
  Test (Testee   => "'\n7'",  -- no octal bit pattern
        Expected => (1 => (Lexer.Bad_Token_AJ , + "'\"     , Lexer.Non_Language_Character),
                     2 => (Lexer.Identifier_AJ, + "n7"),
                     3 => (Lexer.Bad_Token_AJ , + "'"      , Lexer.Non_Language_Character)));
  --Test (Testee   => "'\xef'",  -- hexadecimal bit pattern
  --      Expected => (1 => (Lexer.Character_AJ, + "'\xef'")));
  --Test (Testee   => "'\u1234'",  -- unicode character
  --      Expected => (1 => (Lexer.Character_AJ, + "'\u1234'")));

  -- String literals

  --Test (Testee   => """string with special Java characters""",
  --      Expected => (1 => (Lexer.String_AJ, + """""")));

  -- Illegal integers with underlines

  Test (Testee   => "2_2",
        Expected => (1 => (Lexer.Bad_Token_AJ, + "2_2", Lexer.Illegal_Underline)));

  -- Zeros (0, 00, 0x0 all represent the same number, JRM 3.10.1)

  Test (Testee   => "0",
        Expected => (1 => (Lexer.Integer_AJ        , + "0")));
  Test (Testee   => "0L",
        Expected => (1 => (Lexer.LongInteger_J     , + "0L")));
  Test (Testee   => "00",
        Expected => (1 => (Lexer.Based_Integer_AJ  , + "00"  , Base =>  8)));
  Test (Testee   => "00L",
        Expected => (1 => (Lexer.BasedLongInteger_J, + "00L" , Base =>  8)));
  Test (Testee   => "0x0",
        Expected => (1 => (Lexer.Based_Integer_AJ  , + "0x0" , Base => 16)));
  Test (Testee   => "0x0L",
        Expected => (1 => (Lexer.BasedLongInteger_J, + "0x0L", Base => 16)));

  -- Long integers

  Test (Testee   => "2L",
        Expected => (1 => (Lexer.LongInteger_J, + "2L")));

  -- Based integers

   Test (Testee   => "0777",  -- octal
         Expected => (1 => (Lexer.Based_Integer_AJ, + "0777"      , Base => 8)));
   Test (Testee   => "08",
         Expected => (1 => (Lexer.Bad_Token_AJ    , + "08"        , Lexer.Illegal_Extended_Digit)));
   Test (Testee   => "0xDeadBeef",  -- hexadecimal
         Expected => (1 => (Lexer.Based_Integer_AJ, + "0xDeadBeef", Base => 16)));
   Test (Testee   => "0xDeadBear",
         Expected => (1 => (Lexer.Based_Integer_AJ, + "0xDeadBea" , Base => 16),
                      2 => (Lexer.Identifier_AJ   , + "r")));
   Test (Testee   => "0xl",
         Expected => (1 => (Lexer.Bad_Token_AJ    , + "0xl"       , Lexer.Illegal_Literal)));

  Test (Testee   => " 3#12#",  -- Ada
        Expected => (1 => (Lexer.Integer_AJ   , + "3"),
                     2 => (Lexer.Bad_Token_AJ , + "#"   , Lexer.Non_Language_Character),
                     3 => (Lexer.Integer_AJ   , + "12"),
                     4 => (Lexer.Bad_Token_AJ , + "#"   , Lexer.Non_Language_Character)));
  Test (Testee   => " 3#12.0#e5",  -- Ada
        Expected => (1 => (Lexer.Integer_AJ   , + "3"),
                     2 => (Lexer.Bad_Token_AJ , + "#"   , Lexer.Non_Language_Character),
                     3 => (Lexer.Real_AJ      , + "12.0"),
                     4 => (Lexer.Bad_Token_AJ , + "#"   , Lexer.Non_Language_Character),
                     5 => (Lexer.Identifier_AJ, + "e5")));

  -- Reals without decimal dot, but with exponents

  Test (Testee   => "0123e10",
        Expected => (1 => (Lexer.Real_AJ, + "0123e10")));
  Test (Testee   => "08e-10",
        Expected => (1 => (Lexer.Real_AJ, + "08e-10")));

  -- Lazy reals without exponents

   Test (Testee   => ".45",
         Expected => (1 => (Lexer.Real_AJ, + ".45")));
   Test (Testee   => "123.",
         Expected => (1 => (Lexer.Real_AJ, + "123.")));

  -- Float and Double

  Test (Testee   => "1F",
        Expected => (1 => (Lexer.FloatNumber_J , + "1F")));
  Test (Testee   => "8d",
        Expected => (1 => (Lexer.DoubleNumber_J, + "8d")));

  Test (Testee   => "123e10F",
        Expected => (1 => (Lexer.FloatNumber_J , + "123e10F")));
  Test (Testee   => "8e-10d",
        Expected => (1 => (Lexer.DoubleNumber_J, + "8e-10d")));
  Test (Testee   => "123eF",
        Expected => (1 => (Lexer.Bad_Token_AJ  , + "123eF" , Lexer.Illegal_Literal)));

   Test (Testee   => ".45E-7f",
         Expected => (1 => (Lexer.FloatNumber_J , + ".45D-7f")));
   Test (Testee   => "123.e+8D",
         Expected => (1 => (Lexer.DoubleNumber_J, + "123.d+8D")));

  Test (Testee   => "1e10L",
        Expected => (1 => (Lexer.Real_AJ      , + "1e10"),
               2 => (Lexer.Identifier_AJ, + "L")));

end Test_Java;
