------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 1998 Christoph Karl Walter Grein
------------------------------------------------------------------------

separate (Test_Lexer)

procedure Test_Keywords is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.1
  -- Date      22 September 1998
  --====================================================================
  -- Test program for Lexer to see whether keywords and identifiers are
  -- properly distinguished.
  -- Keywords: Reserved words and Java boolean and null literals.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  29.07.1998
  --  C.G.    1.1  22.09.1998 Clean up
  --====================================================================

  procedure Test (Testee: in String; Language: in String; Requested: in Boolean) is
    Token: Lexer.Token;
    use type Lexer.Token_Name;
  begin
    Lexer.Initialize (new String'(Language & Testee), Size_of_Tab => 8);
    Token := Lexer.Get_Token;  -- ignore language deciding first token
    Token := Lexer.Get_Token;
    if Requested then
      if Lexer.Name (Token) not in Lexer.Reserved_Word and
         Lexer.Name (Token) not in Lexer.Null_J .. Lexer.True_J then
        Number_of_Errors         := Number_of_Errors + 1;
        Number_of_Keyword_Errors := Number_of_Keyword_Errors + 1;
        Ada.Text_IO.Put_Line (Testee & " not recognized as keyword");
        Ada.Text_IO.New_Line;
      end if;
    elsif Lexer.Name (Token) /= Lexer.Identifier_AJ or Lexer.Image (Token) /= Testee then
      Number_of_Errors         := Number_of_Errors + 1;
      Number_of_Keyword_Errors := Number_of_Keyword_Errors + 1;
      Ada.Text_IO.Put_Line (Testee & " not correctly recognized as identifier");
      Ada.Text_IO.Put_Line (Lexer.Token_Name'Image (Lexer.Name (Token)) & ' ' & Lexer.Image (Token));
      Ada.Text_IO.New_Line;
    end if;
  end Test;

  package Text_IO renames Ada.Text_IO;  -- to prevent name clash

  type Language is (Ada, Java);

  type Language_Pointer is access constant String;

  Language_String: constant array (Language) of Language_Pointer :=
    (Ada  => Ada_Language 'Unchecked_Access,
     Java => Java_Language'Unchecked_Access);

  type Allowed is array (Language) of Boolean;

  type Descriptor is record
    Image: Lexer.String_Pointer;
    Legal: Allowed;
  end record;

  use Lexer;

  Reserved_Word: constant array (Lexer.Reserved_Word) of Descriptor :=
    (Abort_A        => (new String'("abort"       ), (Ada => True , Java => False)),
     Abs_A          => (new String'("abs"         ), (Ada => True , Java => False)),
     Abstract_AJ    => (new String'("abstract"    ), (Ada => True , Java => True )),
     Accept_A       => (new String'("accept"      ), (Ada => True , Java => False)),
     Access_A       => (new String'("access"      ), (Ada => True , Java => False)),
     Aliased_A      => (new String'("aliased"     ), (Ada => True , Java => False)),
     All_A          => (new String'("all"         ), (Ada => True , Java => False)),
     And_A          => (new String'("and"         ), (Ada => True , Java => False)),
     Array_A        => (new String'("array"       ), (Ada => True , Java => False)),
     At_A           => (new String'("at"          ), (Ada => True , Java => False)),
     Begin_A        => (new String'("begin"       ), (Ada => True , Java => False)),
     Body_A         => (new String'("body"        ), (Ada => True , Java => False)),
     Boolean_J      => (new String'("boolean"     ), (Ada => False, Java => True )),
     Break_J        => (new String'("break"       ), (Ada => False, Java => True )),
     Byte_J         => (new String'("byte"        ), (Ada => False, Java => True )),
     Case_AJ        => (new String'("case"        ), (Ada => True , Java => True )),
     Catch_J        => (new String'("catch"       ), (Ada => False, Java => True )),
     Char_J         => (new String'("char"        ), (Ada => False, Java => True )),
     Class_J        => (new String'("class"       ), (Ada => False, Java => True )),
     Const_J        => (new String'("const"       ), (Ada => False, Java => True )),
     Constant_A     => (new String'("constant"    ), (Ada => True , Java => False)),
     Continue_J     => (new String'("continue"    ), (Ada => False, Java => True )),
     Declare_A      => (new String'("declare"     ), (Ada => True , Java => False)),
     Default_J      => (new String'("default"     ), (Ada => False, Java => True )),
     Delay_A        => (new String'("delay"       ), (Ada => True , Java => False)),
     Delta_A        => (new String'("delta"       ), (Ada => True , Java => False)),
     Digits_A       => (new String'("digits"      ), (Ada => True , Java => False)),
     Do_AJ          => (new String'("do"          ), (Ada => True , Java => True )),
     Double_J       => (new String'("double"      ), (Ada => False, Java => True )),
     Else_AJ        => (new String'("else"        ), (Ada => True , Java => True )),
     Elsif_A        => (new String'("elsif"       ), (Ada => True , Java => False)),
     End_A          => (new String'("end"         ), (Ada => True , Java => False)),
     Entry_A        => (new String'("entry"       ), (Ada => True , Java => False)),
     Exception_A    => (new String'("exception"   ), (Ada => True , Java => False)),
     Exit_A         => (new String'("exit"        ), (Ada => True , Java => False)),
     Extends_J      => (new String'("extends"     ), (Ada => False, Java => True )),
     Final_J        => (new String'("final"       ), (Ada => False, Java => True )),
     Finally_J      => (new String'("finally"     ), (Ada => False, Java => True )),
     Float_J        => (new String'("float"       ), (Ada => False, Java => True )),
     For_AJ         => (new String'("for"         ), (Ada => True , Java => True )),
     Function_A     => (new String'("function"    ), (Ada => True , Java => False)),
     Generic_A      => (new String'("generic"     ), (Ada => True , Java => False)),
     Goto_AJ        => (new String'("goto"        ), (Ada => True , Java => True )),
     If_AJ          => (new String'("if"          ), (Ada => True , Java => True )),
     Implements_J   => (new String'("implements"  ), (Ada => False, Java => True )),
     Import_J       => (new String'("import"      ), (Ada => False, Java => True )),
     In_A           => (new String'("in"          ), (Ada => True , Java => False)),
     InstanceOf_J   => (new String'("instanceof"  ), (Ada => False, Java => True )),
     Int_J          => (new String'("int"         ), (Ada => False, Java => True )),
     Interface_J    => (new String'("interface"   ), (Ada => False, Java => True )),
     Is_A           => (new String'("is"          ), (Ada => True , Java => False)),
     Limited_A      => (new String'("limited"     ), (Ada => True , Java => False)),
     Long_J         => (new String'("long"        ), (Ada => False, Java => True )),
     Loop_A         => (new String'("loop"        ), (Ada => True , Java => False)),
     Mod_A          => (new String'("mod"         ), (Ada => True , Java => False)),
     Native_J       => (new String'("native"      ), (Ada => False, Java => True )),
     New_AJ         => (new String'("new"         ), (Ada => True , Java => True )),
     Not_A          => (new String'("not"         ), (Ada => True , Java => False)),
     Null_A         => (new String'("null"        ), (Ada => True , Java => True )),  -- Java: recognized as Null_J
     Of_A           => (new String'("of"          ), (Ada => True , Java => False)),
     Or_A           => (new String'("or"          ), (Ada => True , Java => False)),
     Others_A       => (new String'("others"      ), (Ada => True , Java => False)),
     Out_A          => (new String'("out"         ), (Ada => True , Java => False)),
     Package_AJ     => (new String'("package"     ), (Ada => True , Java => True )),
     Pragma_A       => (new String'("pragma"      ), (Ada => True , Java => False)),
     Private_AJ     => (new String'("private"     ), (Ada => True , Java => True )),
     Procedure_A    => (new String'("procedure"   ), (Ada => True , Java => False)),
     Protected_AJ   => (new String'("protected"   ), (Ada => True , Java => True )),
     Public_J       => (new String'("public"      ), (Ada => False, Java => True )),
     Raise_A        => (new String'("raise"       ), (Ada => True , Java => False)),
     Range_A        => (new String'("range"       ), (Ada => True , Java => False)),
     Record_A       => (new String'("record"      ), (Ada => True , Java => False)),
     Rem_A          => (new String'("rem"         ), (Ada => True , Java => False)),
     Renames_A      => (new String'("renames"     ), (Ada => True , Java => False)),
     Requeue_A      => (new String'("requeue"     ), (Ada => True , Java => False)),
     Return_AJ      => (new String'("return"      ), (Ada => True , Java => True )),
     Reverse_A      => (new String'("reverse"     ), (Ada => True , Java => False)),
     Select_A       => (new String'("select"      ), (Ada => True , Java => False)),
     Separate_A     => (new String'("separate"    ), (Ada => True , Java => False)),
     Short_J        => (new String'("short"       ), (Ada => False, Java => True )),
     Static_J       => (new String'("static"      ), (Ada => False, Java => True )),
     Subtype_A      => (new String'("subtype"     ), (Ada => True , Java => False)),
     Super_J        => (new String'("super"       ), (Ada => False, Java => True )),
     Switch_J       => (new String'("switch"      ), (Ada => False, Java => True )),
     Synchronized_J => (new String'("synchronized"), (Ada => False, Java => True )),
     Tagged_A       => (new String'("tagged"      ), (Ada => True , Java => False)),
     Task_A         => (new String'("task"        ), (Ada => True , Java => False)),
     Terminate_A    => (new String'("terminate"   ), (Ada => True , Java => False)),
     Then_A         => (new String'("then"        ), (Ada => True , Java => False)),
     This_J         => (new String'("this"        ), (Ada => False, Java => True )),
     Throw_J        => (new String'("throw"       ), (Ada => False, Java => True )),
     Throws_J       => (new String'("throws"      ), (Ada => False, Java => True )),
     Transient_J    => (new String'("transient"   ), (Ada => False, Java => True )),
     Try_J          => (new String'("try"         ), (Ada => False, Java => True )),
     Type_A         => (new String'("type"        ), (Ada => True , Java => False)),
     Until_A        => (new String'("until"       ), (Ada => True , Java => False)),
     Use_A          => (new String'("use"         ), (Ada => True , Java => False)),
     Void_J         => (new String'("void"        ), (Ada => False, Java => True )),
     Volatile_J     => (new String'("volatile"    ), (Ada => False, Java => True )),
     When_A         => (new String'("when"        ), (Ada => True , Java => False)),
     While_AJ       => (new String'("while"       ), (Ada => True , Java => True )),
     With_A         => (new String'("with"        ), (Ada => True , Java => False)),
     Xor_A          => (new String'("xor"         ), (Ada => True , Java => False)));

  Keyword: constant array (Lexer.Null_J .. Lexer.True_J) of Descriptor :=
    (False_J => (new String'("false"), (Ada => False, Java => True)),
     Null_J  => (new String'("null" ), (Ada => True , Java => True)),  -- Ada: recognized as Null_A
     True_J  => (new String'("true" ), (Ada => False, Java => True)));

begin

  Text_IO.Put_Line ("Keywords test");
  Text_IO.Put_Line ("=============");
  Text_IO.New_Line;

  for Each in Language loop
    Text_IO.Put_Line ("Language is " & Language'Image (Each));
    Text_IO.New_Line;
    for This in Reserved_Word'Range loop
      Test (Reserved_Word (This).Image.all, Language_String (Each).all, Reserved_Word (This).Legal (Each));
    end loop;
    for This in Lexer.Null_J .. Lexer.True_J loop
      Test (Keyword (This).Image.all, Language_String (Each).all, Keyword (This).Legal (Each));
    end loop;
  end loop;

end Test_Keywords;
